
{*******************************************************}
{                                                       }
{       SAB Object Framework                            }
{       SAB RESTsend                                    }
{                                                       }
{       Copyright (c) 2017-22 TDP                       }
{                                                       }
{****************************************************tk*}
{==============================================================================|
| Project : Ararat Synapse                                       | 003.012.009 |
|==============================================================================|
| Content: HTTP client                                                         |
|==============================================================================|
| Copyright (c)1999-2015, Lukas Gebauer                                        |
| All rights reserved.                                                         |
|                                                                              |
| Redistribution and use in source and binary forms, with or without           |
| modification, are permitted provided that the following conditions are met:  |
|                                                                              |
| Redistributions of source code must retain the above copyright notice, this  |
| list of conditions and the following disclaimer.                             |
|                                                                              |
| Redistributions in binary form must reproduce the above copyright notice,    |
| this list of conditions and the following disclaimer in the documentation    |
| and/or other materials provided with the distribution.                       |
|                                                                              |
| Neither the name of Lukas Gebauer nor the names of its contributors may      |
| be used to endorse or promote products derived from this software without    |
| specific prior written permission.                                           |
|                                                                              |
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  |
| AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    |
| IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   |
| ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR  |
| ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       |
| DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR   |
| SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER   |
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT           |
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    |
| OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  |
| DAMAGE.                                                                      |
|==============================================================================|
| The Initial Developer of the Original Code is Lukas Gebauer (Czech Republic).|
| Portions created by Lukas Gebauer are Copyright (c) 1999-2015.               |
| All Rights Reserved.                                                         |
|==============================================================================|
| Contributor(s):                                                              |
|==============================================================================}

unit SAB.RESTsend;

{$IFNDEF USE_SSL_0_9}
  {$DEFINE SSL_1_1_1}
{$ENDIF}

interface

{ DEFINE READ_PROXY_SETTINGS}

uses
  WinApi.Windows, System.SysUtils, System.Classes,
  httpsend;

{ DEFINE USE_CRITICAL_SECTION}

type
  TRESTsend = class(THTTPsend)
  private
    FAccessCode: string;
    FProxyOverride: string;
    FAutoConfigURL: string;
    FProxyEnabled: Boolean;
    FTextResponse: Boolean;

{$IFDEF USE_CRITICAL_SECTION}
    FCritSec: TRTLCriticalSection;
{$ENDIF}
    FStoreTimeout: Integer;

    procedure InitCmd(const AResponseContent: TBytesStream; const UseAccessCode: Boolean = True);
{$IFDEF READ_PROXY_SETTINGS}
    procedure ReadProxySettings(const AOwner: TObject);
{$ENDIF}
  public
    constructor Create(AOwner: TObject; const AAccessCode: string = '');
    destructor Destroy; override;

    function PostFile(const URI: string; const FieldName, FileName: string; Stream: TStream; const AResponseContent: TBytesStream = nil): Boolean;
    function GetFile(const URI: string; const Source: TStringList; const AResponse: TBytesStream): Boolean;
    function Options(const URI: string; const Source: TStringList; const AResponseContent: TBytesStream = nil; const PostUrl: Boolean = False): Boolean;
    function Post(const URI: string; const Source: TBytesStream; const AResponseContent: TBytesStream = nil): Boolean; overload;
    function Post(const URI: string; const Source: TStringList; const AResponseContent: TBytesStream = nil): Boolean; overload;
    function Put(const URI: string; const Source: TBytesStream; const AResponseContent: TBytesStream = nil; const UseAccessCode: Boolean = True): Boolean;
    function Get(const URI: string; const AResponseContent: TBytesStream; const UseAccessCode: Boolean = True): Boolean;
    function GetPage(const URI: string): Boolean;
    function Delete(const URI: string; const AResponseContent: TBytesStream = nil): Boolean; overload;
    function Delete(const URI: string; const ASource, AResponseContent: TBytesStream): Boolean; overload;

{$IFDEF CPUX86}
    procedure CancelTraffic;
{$ENDIF}
    procedure RestoreTraffic;

    property AccessCode: string read FAccessCode write FAccessCode;

    property ProxyEnabled: Boolean read FProxyEnabled;
    property ProxyOverride: string read FProxyOverride;
    property AutoConfigURL: string read FAutoConfigURL;

    property TextResponse: Boolean read FTextResponse write FTextResponse;
  end;

function EncodePswdTriplet(const Value: string): string;
function EncodeGetQrTriplet(const Value: string): string;
function EncodeListItem(const Value: string): string;
function EncodeUnicodeURL(const Value: string): string;

implementation

uses
  System.Win.Registry,
  blcksock, synacode, synautil, 
{$IFDEF SSL_1_1_1}
  ssl_openssl11,
{$ELSE}
  ssl_openssl,
{$ENDIF}
  SAB.Clients, SAB.OpenSSL;

const
  PROXY_PATH       = 'Software\Microsoft\Windows\CurrentVersion\Internet Settings';

  PROXY_SERVER     = 'ProxyServer';
  PROXY_OVERRIDE   = 'ProxyOverride';
  PROXY_ENABLE     = 'ProxyEnable';
  AUTO_CONFIG_URL  = 'AutoConfigURL';

{ Unicode decoding/encoding URL }

function EncodePswdTriplet(const Value: string): string;
var
  US: UTF8String;
begin
  US := UTF8Encode(Value);
  Result := string(EncodeTriplet(AnsiString(US), '%', [' ', '(', ')', ':', #$7F..#$FF]));
end;

// see, double encoding (%)
function EncodeGetQrTriplet(const Value: string): string;
var
  US: UTF8String;
begin
  US := UTF8Encode(Value);
  Result := string(EncodeTriplet(AnsiString(US), '%', [' ', '%', '(', ')', ':', '/', '&', '=', '?', #$7F..#$FF]));
end;

function EncodeUnicodeURL(const Value: string): string;
var
  US: UTF8String;
begin
  US := UTF8Encode(Value);
  Result := string(EncodeTriplet(AnsiString(US), '%', [' ', #$7F..#$FF]));
end;

function EncodeListItem(const Value: string): string;
var
  US: UTF8String;
begin
  US := UTF8Encode(Value);
  Result := string(EncodeURLElement(AnsiString(US)));
end;

{ TRESTsend }

constructor TRESTsend.Create(AOwner: TObject; const AAccessCode: string);
var
  Client: TSabClient;
begin
  inherited Create;

{$IFDEF USE_CRITICAL_SECTION}
  InitializeCriticalSection(FCritSec);
{$ENDIF}

  Protocol := '1.1';
  FAccessCode := AAccessCode;

  if Assigned(AOwner) and (AOwner is TSabClient) then
  begin
    Client := TSabClient(AOwner);
    if Client.ConnectionTimeout <> 0 then
      Sock.ConnectionTimeout := Client.ConnectionTimeout;

    if Assigned(Sock.SSL) then
    begin
      Sock.SSL.VerifyCert := Client.VerifyCertificate;
      Sock.SSL.CertificateFile := Client.CertificateFile;
      Sock.SSL.Certificate := Client.Certificate;
      Sock.SSL.CertCAFile := Client.CertCAFile;
      Sock.SSL.CertCA := Client.CertCA;
      Sock.SSL.PFX := Client.PFX;
      if Sock.SSL is TSabOpenSSL then
      begin
        TSabOpenSSL(Sock.SSL).CertCABytes := Client.CertCABytes;
        TSabOpenSSL(Sock.SSL).Imprint := Client.Imprint;
      end;
    end;
  end;

{$IFDEF READ_PROXY_SETTINGS}
  ReadProxySettings(AOwner);
{$ENDIF}
end;

destructor TRESTsend.Destroy;
begin
{$IFDEF USE_CRITICAL_SECTION}
  DeleteCriticalSection(FCritSec);
{$ENDIF}
  inherited;
end;

{$IFDEF CPUX86}
procedure TRESTsend.CancelTraffic;
begin
  if Timeout > 0 then
  begin
    asm
           XOR  ECX, ECX
           MOV  EAX, Self
      LOCK XCHG [EAX].FTimeout, ECX
           MOV  [EAX].FStoreTimeout, ECX
    end;
  end;
end;
{$ENDIF}

function TRESTsend.Delete(const URI: string; const AResponseContent: TBytesStream): Boolean;
begin
  InitCmd(AResponseContent);

  Result := HTTPMethod('DELETE', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

function TRESTsend.Delete(const URI: string; const ASource, AResponseContent: TBytesStream): Boolean;
begin
  InitCmd(AResponseContent);

  Document.CopyFrom(ASource);
  Result := HTTPMethod('DELETE', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

function TRESTsend.Get(const URI: string; const AResponseContent: TBytesStream; const UseAccessCode: Boolean): Boolean;
begin
  InitCmd(AResponseContent, UseAccessCode);

  Result := HTTPMethod('GET', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

function TRESTsend.GetFile(const URI: string;  const Source: TStringList; const AResponse: TBytesStream): Boolean;
var
  I: Integer;
  VS, NSS, Doc: AnsiString;
begin
  InitCmd(nil);

  if Assigned(Source) then
  begin
    Doc := '';
    for I := 0 to Source.Count - 1 do
    begin
      NSS := AnsiString(Source.Names[I]) + AnsiString(Source.NameValueSeparator);
      VS := AnsiString(EncodeListItem(Source.ValueFromIndex[I]));
      if Doc = '' then
        Doc := NSS + VS
      else
        Doc := Doc + '&' + NSS + VS;
    end;
    Document.WriteData(TBytes(Doc), Length(Doc));
  end;

  Result := HTTPMethod('POST', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponse) then
    AResponse.LoadFromStream(Document);
end;

function TRESTsend.GetPage(const URI: string): Boolean;
begin
  InitCmd(nil);

  Result := HTTPMethod('GET', EncodeUnicodeURL(URI));
end;

procedure TRESTsend.InitCmd(const AResponseContent: TBytesStream; const UseAccessCode: Boolean);
begin
  Document.Clear;
  Headers.Clear;

  if FTextResponse then
  begin
    Headers.Add('Accept: text/plain');
    FTextResponse := False;
  end else
    Headers.Add('Accept: application/json');

  if UseAccessCode and (FAccessCode <> '') then
    Headers.Add('SAB-Access-Code: ' + EncodeListItem(FAccessCode));

  if Assigned(AResponseContent) then
    AResponseContent.Clear;
end;

function TRESTsend.Options(const URI: string; const Source: TStringList; const AResponseContent: TBytesStream; const PostUrl: Boolean): Boolean;
var
  I: Integer;
  US, Doc, Key: AnsiString;
begin
  InitCmd(nil);
  if Assigned(Source) then
  begin
    Doc := '';
    for I := 0 to Source.Count - 1 do
    begin
      Key := AnsiString(Source.KeyNames[I]);
      US := UTF8Encode(Source.ValueFromIndex[I]);
      if Doc = '' then
        Doc := Key + '=' + EncodeURLElement(US)
      else
        Doc := Doc + '&' + Key + '=' + EncodeURLElement(US);
    end;
    Document.WriteData(TBytes(Doc), Length(Doc));
  end;

  if PostUrl then
    MimeType := 'application/x-www-form-urlencoded';
  Result := HTTPMethod('OPTIONS', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

function TRESTsend.Post(const URI: string; const Source: TBytesStream; const AResponseContent: TBytesStream): Boolean;
begin
  InitCmd(AResponseContent);
  if Assigned(Source) then
    Document.CopyFrom(Source, -1);

  Result := HTTPMethod('POST', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

function TRESTsend.Post(const URI: string; const Source: TStringList; const AResponseContent: TBytesStream): Boolean;
var
  I: Integer;
  US, Doc, Key: AnsiString;
begin
  InitCmd(nil);
  if Assigned(Source) then
  begin
    Doc := '';
    for I := 0 to Source.Count - 1 do
    begin
      Key := AnsiString(Source.KeyNames[I]);
      US := UTF8Encode(Source.ValueFromIndex[I]);
      if Doc = '' then
        Doc := Key + '=' + EncodeURLElement(US)
      else
        Doc := Doc + '&' + Key + '=' + EncodeURLElement(US);
    end;
    Document.WriteData(TBytes(Doc), Length(Doc));
  end;

  Result := HTTPMethod('POST', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

function TRESTsend.PostFile(const URI: string; const FieldName, FileName: string; Stream: TStream; const AResponseContent: TBytesStream): Boolean;
var
  Bound, S: string;
  US: AnsiString;
begin
  InitCmd(nil);
  if RandSeed = 0 then
    Randomize;

  Bound := IntToHex(Random(MaxInt - 1), 8) + '_Synapse_boundary';

  S := '--' + Bound + CRLF;
  S := S + 'content-disposition: form-data; name="' + FieldName + '";';
  S := S + ' filename="' + FileName +'"' + CRLF;
  S := S + 'Content-Type: Application/octet-string' + CRLF + CRLF;
  US := UTF8Encode(S);
  WriteStrToStream(Document, US);

  Document.CopyFrom(Stream, 0);

  US := UTF8Encode(CRLF + '--' + Bound + '--' + CRLF);
  WriteStrToStream(Document, US);

  MimeType := 'multipart/form-data; boundary=' + Bound;

  Result := HTTPMethod('POST', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.LoadFromStream(Document);
end;

function TRESTsend.Put(const URI: string; const Source: TBytesStream; const AResponseContent: TBytesStream; const UseAccessCode: Boolean): Boolean;
begin
  InitCmd(AResponseContent, UseAccessCode);

  if Assigned(Source) then
    Document.CopyFrom(Source, -1);

  Result := HTTPMethod('PUT', EncodeUnicodeURL(URI));

  if Result and Assigned(AResponseContent) then
    AResponseContent.CopyFrom(Document, -1);
end;

procedure TRESTsend.RestoreTraffic;
begin
{$IFDEF USE_CRITICAL_SECTION}
  EnterCriticalSection(FCritSec);
  try
{$ENDIF}
    if FStoreTimeout > 0 then
      FTimeout := FStoreTimeout;
    FStoreTimeout := 0;
{$IFDEF USE_CRITICAL_SECTION}
  finally
    LeaveCriticalSection(FCritSec);
  end;
{$ENDIF}
end;

{$IFDEF READ_PROXY_SETTINGS}
procedure TRESTsend.ReadProxySettings(const AOwner: TObject);
var
  Reg: TRegistry;
  Client: TSabClient;
  PXS: string;
  User, Pass, Protcl, Host, Port, Path, Param: string;
begin
  Reg := TRegistry.Create(KEY_READ);
  try
    Reg.RootKey := HKEY_CURRENT_USER;

    if Reg.OpenKey(PROXY_PATH, False) then
    begin
      FProxyEnabled := Reg.ReadInteger(PROXY_ENABLE) <> 0;
      FAutoConfigURL := Reg.ReadString(AUTO_CONFIG_URL);
      if FProxyEnabled then
      begin
        PXS := Reg.ReadString(PROXY_SERVER);

        Client := TSabClient(AOwner);
        Client.ParseURL(PXS, Protcl, User, Pass, Host, Port, Path, Param, False);

        ProxyHost := Protcl + Host;
        ProxyPort := Port;

        FProxyOverride := Reg.ReadString(PROXY_OVERRIDE);
      end;
    end;
  finally
    Reg.Free;
  end;
end;
{$ENDIF}

end.
