
{*******************************************************}
{                                                       }
{       SAB Object Framework                            }
{       SAB Client Uri                                  }
{                                                       }
{       Copyright (c) 2017-23 TDP                       }
{                                                       }
{****************************************************tk*}

unit SAB.Uri;

{ DEFINE PREV_SAB_4_17_1}

interface

uses
  System.SysUtils, System.Classes,
  Sab.Types;

type
  TGenerationType = (gtPassword, gtAccessCode);
  TWatchedType = (wtChanges, wtAccesses);
  TWatchedTypes = set of TWatchedType;

  TSabUri = class(TObject)
  strict private
    function URI_GetUpdateRecordPasswords(const ARecId: TSabId): string;
    function URI_GetDelUpdateRecord(const ARecId: TSabId): string;
  protected
    function URI_Login: string;
    function URI_CertLogin: string;
    function URI_RegisterSecondFactor: string;
    function URI_GetSecondFactorQrImage(const APath: string): string;
    function URI_SecondFactor: string;
    function URI_LoginStatus: string;
    function URI_DeleteRecord(const ARecId: TSabId): string;
    function URI_GetStatus: string;
    function URI_ReadDeleteSafeBox(const ABoxId: TSabId): string;
    function URI_UpdateSafeBox(const ABoxId: TSabId): string;
    function URI_ReadSafeBoxWithExtPath(const ABoxId: TSabId): string;
    function URI_ReadSafeBoxes(const AParentBoxId: TSabId): string;
    function URI_ReadRecords(const ABoxId: TSabId; const GetFileSizes: Boolean = False): string;
    function URI_ReadRecord(const ARecId: TSabId): string;
    function URI_ReadRecordForm(const ARecId: TSabId): string;
    function URI_ReadRecordPasswords(const ARecId: TSabId): string;
    function URI_ChangePasswords(const ARecId: TSabId): string;
    function URI_AddBox(ABoxId: TSabId): string;
    function URI_AddRecord(ABoxId: TSabId): string;
    function URI_UpdateRecord(const ARecId: TSabId): string;
    function URI_UploadFileToRecord(ARecId: TSabId): string;
    function URI_DownloadFileFromRecord(ARecId: TSabId): string;
    function URI_ReadSecureAttrs(ARecId: TSabId): string;
    function URI_GetStations(const AConfiguration: string): string;
    function URI_GetConfigurations: string;
    function URI_GetAgentPassword(const AName, AConfiguration, APlatform, AUser: string; ADate: TDateTime): string;
    function URI_GetPassword(const AName, AConfiguration, APlatform, AUser: string; ADate: TDateTime): string;
    function URI_GetTicketPassword(const ATicketId, AStation, APlatform, AUser: string): string;
    function URI_GetExternalFileParams(const ABoxId: TSabId): string;

    function URI_ReadRemoveRestoreDeletedRecords: string;

    function URI_Search(const APattern: string): string;
    function URI_SearchByNote(const APattern: string): string;

    function URI_SetWatched(const ABoxId: TSabId; AWatchType: TWatchedType; All, Enable: Boolean): string;

    function URI_CheckPswdValidity: string;
    function URI_GeneratePswdOrAC(AType: TGenerationType; const ACode: string): string;
    function URI_GetUserConfiguration(const ACode: string): string;

    function URI_SetPasswordToInvitedUser(const ACode: string): string;
    function URI_SetUserNewAccessCode: string;

    function URI_GetErrorMessages(ALanguage: TSabLanguage): string;
    function URI_GetFileSizeLimit: string;

    function URI_About: string;
  public
    class function ParseURL(const URL: string; out AProtocol, AUser, APass, AHost, APort, APath, AParam: string; const ASecure: Boolean = True): string;
    class function PrepareURL(const AUrl: string; AddProtocol: Boolean = True; AddPort: Boolean = False; const ASecure: Boolean = True): string;
  end;

implementation

uses
  System.StrUtils,
  synautil;

const
  PUBLIC_RESOURCES                        = 'public';
  PRIVATE_RESOURCES                       = 'private';
  SAFE_RESOURCES                          = 'safe';
  STATIC_PAGES                            = 'static';

  PUBLIC_API                              = PUBLIC_RESOURCES + '/api';
  PRIVATE_API                             = PRIVATE_RESOURCES + '/api';
  SAFE_API                                = 'safe';
  WHITE_API                               = 'whitebox';
  BASE_API                                = 'api';
  AGENT_API                               = 'agent';

  ABOUT_PAGE                              = STATIC_PAGES + '/about.page';
  GET_STATUS                              = PUBLIC_API + '/status';

  API_MESSAGES                            = PUBLIC_API + '/messages';
  API_MESSAGES_EN                         = API_MESSAGES + '/en';
  API_MESSAGES_CS                         = API_MESSAGES + '/cs';
  API_MESSAGES_FR                         = API_MESSAGES + '/fr';
  API_MESSAGES_DE                         = API_MESSAGES + '/de';

  PRIVATE_API_CONFIG_PATH                 = PRIVATE_API + '/config';
  PRIVATE_API_TICKETS_PATH                = PRIVATE_API + '/tickets';
  PRIVATE_API_SERVICE_PATH                = PRIVATE_API + '/service';
  PRIVATE_API_DOMAINS_PATH                = PRIVATE_API + '/domains';
  PRIVATE_API_USERS_PATH                  = PRIVATE_API + '/users';
  PRIVATE_API_LDAP_PATH                   = PRIVATE_API + '/ldap';
  PRIVATE_API_LDAP_CONNECTOR_PATH         = PRIVATE_API + '/connector';
  PRIVATE_API_CONVERT_TIMEZONE_PATH       = PRIVATE_API + '/convertTZ';
  PRIVATE_API_STATION_PASSWORD_PATH       = PRIVATE_API + '/stationPassword';
  API_GET_STATION_PASSWORD                = AGENT_API + '/getPassword';
  PRIVATE_API_GENERATE_PASSWORD_PATH      = PRIVATE_API + '/generate-password';
  PRIVATE_API_LOG_PATH                    = PRIVATE_API + '/logview';
  PRIVATE_API_AUDITLOG_PATH               = PRIVATE_API + '/auditlog';
  PRIVATE_API_AGENT_CONFIGS_PATH          = PRIVATE_API + '/agentConfigs';
  PRIVATE_API_AGENT_PATH                  = PRIVATE_API + '/agent';
  PRIVATE_API_LDAP_AGENT_PATH             = PRIVATE_API + '/ldapAgent/{agentName}';
  PRIVATE_API_STATIONS_PATH               = PRIVATE_API + '/stations';

  API_SAFE_BOXES_PATH                     = SAFE_API + '/boxes';

  API_SAFEBOX_PATH                        = API_SAFE_BOXES_PATH + '/%d';                // safeBoxId
  API_SAFEBOX_BOXES_PATH                  = API_SAFEBOX_PATH + '/boxes';
  API_SAFEBOX_RECORDS_PATH                = API_SAFEBOX_PATH + '/records';
{$IFDEF PREV_SAB_4_17_1}
  API_SAFEBOX_RECORDS_PATH_FILE_SIZES     = API_SAFEBOX_RECORDS_PATH + '?fileSizes=true';
{$ELSE}
  API_SAFEBOX_RECORDS_PATH_FILE_SIZES     = API_SAFEBOX_RECORDS_PATH + '?fileInfo=true';
{$ENDIF}
  API_SAFEBOX_ACL_PATH                    = API_SAFEBOX_PATH + '/perms';
  API_SAFEBOX_EXPORT_PATH                 = API_SAFEBOX_PATH + '/export';
  API_SAFEBOX_MOVE_PATH                   = API_SAFEBOX_PATH + '/move';
  API_SAFEBOX_GEN_EXT_PARAMS              = API_SAFEBOX_PATH + '/generateExternalFileParams';
  API_SAFEBOX_GET_EXT_PATH                = API_SAFEBOX_PATH + '?getExternalFilesPath=true';

  API_RECORDS_PATH                        = SAFE_API + '/records';
  API_RECORD_PATH                         = API_RECORDS_PATH + '/%d';                   // accountId
  API_RECORD_FORM_PATH                    = API_RECORD_PATH + '/form';
  API_RECORD_PASSWORDS_PATH               = API_RECORD_PATH + '/passwords';
  API_RECORD_MOVE_PATH                    = API_RECORD_PATH + '/move';
  API_RECORD_STORE_PATH                   = API_RECORD_PATH + '/store';

//  API_PASSWORD_PATH                       = SAFE_API + '/passwords/%d';                 // passwordId

  API_SAFE_TAGS_PATH                      = SAFE_API + '/tags';
  API_SAFE_WATCH_PATH                     = SAFE_API + '/watch/%d';                     // boxId
  API_SAFE_THRASH_PATH                    = SAFE_API + '/deleted';
  API_SAFE_SEARCH_PATH                    = SAFE_API + '/search/%s';                    // search
  API_SAFE_SEARCH_BY_NOTE_PATH            = SAFE_API + '/searchIdsByNote/%s';           // search in record notes
  API_SAFE_ACTIVATE_ADMINS_PATH           = SAFE_API + '/activateAdmins';

  API_SAFE_REPORTS_PATH                   = SAFE_API + '/reports';
  API_SAFE_REPORTS_USERS_PATH             = API_SAFE_REPORTS_PATH + '/users';
  API_SAFE_REPORTS_BOXES_PATH             = API_SAFE_REPORTS_PATH + '/boxes';

  API_SAFE_USER_ACCESS_REPORT_PATH        = API_SAFE_REPORTS_PATH + '/access-report/%d';            // userId
  API_SAFE_BOX_ACCESS_REPORT_PATH         = API_SAFE_REPORTS_PATH + '/safebox-access-report/%d';    // boxId
  API_SAFE_USER_PERMISSIONS_REPORT_PATH   = API_SAFE_REPORTS_PATH + '/user-permissions-report/%d';  // userId
  API_SAFE_PASSWORD_REPORT_PATH           = API_SAFE_REPORTS_PATH + '/password-report/%d';          // safeBoxId

  API_LOGIN_PATH                          = BASE_API + '/login';                        // also get access code timeout
  API_CERT_LOGIN_PATH                     = BASE_API + '/certlogin';
  API_LOGIN_STATUS_PATH                   = PUBLIC_API + '/loginStatus';                // also get access code timeout

  API_USER_PATH                           = BASE_API + '/user';
  API_USER_AUTH_SECRET_PATH               = API_USER_PATH + '/usersecret';
  API_INIT_USER_SITE_PATH                 = API_USER_PATH + '/initusersite';
  API_USER_2ND_FACTOR_PATH                = API_USER_PATH + '/secondfactor';
  API_USER_REGISTER_2ND_FACTOR_PATH       = API_USER_PATH + '/secondfactor-reg';
  API_GET_2ND_FACTOR_QR_IMAGE             = PUBLIC_API + '/qr?url=%s';

  API_INITADMIN_PATH                      = PUBLIC_API + '/initadmin';
  API_INITUSER_PATH                       = PUBLIC_API + '/inituser';
  API_KSHIELD_NOTIFY_PATH                 = PUBLIC_API + '/kshield/notify';
  API_KSHIELD_POLL_PATH                   = PUBLIC_API + '/kshield/poll';
  API_TICKET_INFO_PATH                    = PUBLIC_API + '/tickets/%d';                 // ticketId

  API_STATUS_PATH                         = PUBLIC_API + '/status';
  API_GET_PASSWORD_PATH                   = PUBLIC_API + '/getPassword';
  API_QR_CODE_PATH                        = PUBLIC_API + '/qr';
  API_REGISTER_STATION_PATH               = PUBLIC_API + '/registerStation';

  API_STORED_RECORD_PATH                  = PUBLIC_API + '/storedRecords/%d';           // storageID

  API_FILEREC_UPLOAD                      = API_RECORD_PATH + '/upload/file';           // fileAttr
  API_FILEREC_DOWNLOAD                    = API_RECORD_PATH + '/download/file';         // fileAttr

  API_CONFIG_STATIONS                     = PRIVATE_API_AGENT_CONFIGS_PATH + '/%s/stations';  // configName
  API_STATION_PASSWORD                    = PRIVATE_API_STATION_PASSWORD_PATH + '?configName=%s&platform=%s&stationName=%s&userName=%s&passwordDate=%s';
  API_AGENT_STATION_PASSWORD              = API_GET_STATION_PASSWORD + '?configName=%s&platform=%s&stationName=%s&userName=%s&passwordDate=%s';

  API_GET_TICKET_PASSWORD_PATH            = PUBLIC_API + '/getPassword';
  API_GET_TICKET_PASSWORD                 = API_GET_TICKET_PASSWORD_PATH + '?ticket=%s&station=%s&platform=%s&user=%s';

  API_CHECK_PASSWORD_VALIDITY             = PUBLIC_API + '/user/generate-password';
  API_GENERATE_PSWD_OR_AC                 = API_CHECK_PASSWORD_VALIDITY + '?code=%s&type=%s';
  API_GENERATE_USER_PSWD_OR_AC            = API_CHECK_PASSWORD_VALIDITY + '?type=%s';
  API_GET_CONFIGURATION                   = PUBLIC_API + '/user/config';
  API_GET_CONFIGURATION_CODE              = PUBLIC_API + '?code=%s';

  API_SET_PASSWORD_TO_INVITED_USER        = API_INITUSER_PATH + '?%s';  // TSabUri.ParseURL returns string 'code=...' in AParam (code=%s');
  API_USER_NEW_ACCESSCODE                 = API_USER_PATH + '/accesscode/new';

  API_FILE_SIZE_LIMIT                     = SAFE_API + '/fileSizeLimit';

  SAFE_WATCH_RESOURCES                    = SAFE_RESOURCES + '/watch/%d';   // box id
  SAFE_WATCH_ENABLE_CHANGES               = SAFE_WATCH_RESOURCES + '/enableChanges';
  SAFE_WATCH_ENABLE_ACCESSES              = SAFE_WATCH_RESOURCES + '/enableAccesses';
  SAFE_WATCH_DISABLE_CHANGES              = SAFE_WATCH_RESOURCES + '/disableChanges';
  SAFE_WATCH_DISBLE_ACCESSES              = SAFE_WATCH_RESOURCES + '/disableAccesses';

  SAFE_WATCH_ENABLE_CHANGES_ALL           = SAFE_WATCH_RESOURCES + '/enableChangesAll';
  SAFE_WATCH_ENABLE_ACCESSES_ALL          = SAFE_WATCH_RESOURCES + '/enableAccessesAll';
  SAFE_WATCH_DISABLE_CHANGES_ALL          = SAFE_WATCH_RESOURCES + '/disableChangesAll';
  SAFE_WATCH_DISBLE_ACCESSES_ALL          = SAFE_WATCH_RESOURCES + '/disableAccessesAll';

  API_GET_DELETED_RECORDS                 = SAFE_API + '/deleted';

// restore trash
// POST: API_SAFE_THRASH_PATH + /ACCOUNT/%d

// clear trash
// POST: API_SAFE_THRASH_PATH
// params boxes (%id, ..)

{ TSabUri }

class function TSabUri.ParseURL(const URL: string; out AProtocol, AUser, APass, AHost, APort, APath, AParam: string; const ASecure: Boolean): string;
var
  X, Y: Integer;
  SURL: string;
  S, S1, S2: string;
begin
  AProtocol := '';
  AUser := '';
  APass := '';
  APort := '';
  AParam := '';

  X := Pos('://', URL);
  if X > 0 then
  begin
    AProtocol := SeparateLeft(URL, '://');
    SURL := SeparateRight(URL, '://');
  end else
    SURL := URL;

  X := Pos('@', SURL);
  Y := Pos('/', SURL);
  if (X > 0) and ((X < Y) or (Y < 1))then
  begin
    S := SeparateLeft(SURL, '@');
    SURL := SeparateRight(SURL, '@');
    X := Pos(':', S);
    if X > 0 then
    begin
      AUser := SeparateLeft(S, ':');
      APass := SeparateRight(S, ':');
    end else
      AUser := S;
  end;

  X := Pos('/', SURL);
  if X > 0 then
  begin
    S1 := SeparateLeft(SURL, '/');
    S2 := SeparateRight(SURL, '/');
  end else
  begin
    S1 := SURL;
    S2 := '';
  end;

  if Pos('[', S1) = 1 then
  begin
    AHost := Separateleft(S1, ']');
    Delete(AHost, 1, 1);
    S1 := SeparateRight(S1, ']');
    if Pos(':', S1) = 1 then
      APort := SeparateRight(S1, ':');
  end else
  begin
    X := Pos(':', S1);
    if X > 0 then
    begin
      AHost := SeparateLeft(S1, ':');
      APort := SeparateRight(S1, ':');
    end else
      AHost := S1;
  end;

  Result := '/' + S2;
  X := Pos('?', S2);
  if X > 0 then
  begin
    APath := '/' + SeparateLeft(S2, '?');
    AParam := SeparateRight(S2, '?');
  end else
    APath := '/' + S2;

  if AHost = '' then
    AHost := 'localhost';
end;

class function TSabUri.PrepareURL(const AUrl: string; AddProtocol, AddPort: Boolean; const ASecure: Boolean): string;
const
  HTTPS_PROT = 'https';
  HTTP_PROT = 'http';
  HTTPS_DEFAULT_PORT = '443';
  HTTP_DEFAULT_PORT = '80';
var
  URI, Protocol, User, Pass, Host, Port, Path, Param: string;
begin
  if AUrl <> '' then
  begin
    URI := ParseURL(AUrl, Protocol, User, Pass, Host, Port, Path, Param, ASecure);

    if Protocol = '' then
      if AddProtocol and (Port = '') then
        if ASecure then
          Protocol := HTTPS_PROT
        else
          Protocol := HTTP_PROT;

    if Port = '' then
      if AddPort then
        if ASecure then
          Port := HTTPS_DEFAULT_PORT
        else
          Port := HTTP_DEFAULT_PORT;

    if Protocol <> '' then
    begin
      if Port <> '' then
        Result := Protocol + '://'+ Host + ':' + Port + URI
      else
        Result := Protocol + '://'+ Host + URI;
    end else
      if Port <> '' then
        Result := Host + ':' + Port + URI
      else
        Result := Host + URI;
  end else
    Result := '';

  if (Result <> '') and (Result[Length(Result)] <> '/') then
    Result := Result + '/';
end;

function TSabUri.URI_About: string;
begin
  Result := ABOUT_PAGE;
end;

function TSabUri.URI_AddBox(ABoxId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_SAFEBOX_PATH, [ABoxId]);
end;

function TSabUri.URI_AddRecord(ABoxId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_SAFEBOX_RECORDS_PATH, [ABoxId]);
end;

function TSabUri.URI_CertLogin: string;
begin
  Result := API_CERT_LOGIN_PATH;
end;

function TSabUri.URI_ChangePasswords(const ARecId: TSabId): string;
begin
  Result := URI_GetUpdateRecordPasswords(ARecId);
end;

function TSabUri.URI_CheckPswdValidity: string;
begin
  Result := API_CHECK_PASSWORD_VALIDITY;
end;

function TSabUri.URI_DeleteRecord(const ARecId: TSabId): string;
begin
  Result := URI_GetDelUpdateRecord(ARecId);
end;

function TSabUri.URI_DownloadFileFromRecord(ARecId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_FILEREC_DOWNLOAD, [ARecId]);
end;

function TSabUri.URI_GeneratePswdOrAC(AType: TGenerationType; const ACode: string): string;
begin
  if AType = gtPassword then
    if ACode = '' then
      Result := System.SysUtils.Format(API_GENERATE_USER_PSWD_OR_AC, ['login'])
    else
      Result := System.SysUtils.Format(API_GENERATE_PSWD_OR_AC, [ACode, 'login'])
  else
    if ACode = '' then
      Result := System.SysUtils.Format(API_GENERATE_USER_PSWD_OR_AC, ['accessCode'])
    else
      Result := System.SysUtils.Format(API_GENERATE_PSWD_OR_AC, [ACode, 'accessCode']);
end;

function TSabUri.URI_GetAgentPassword(const AName, AConfiguration, APlatform, AUser: string; ADate: TDateTime): string;
var
  DT: string;
begin
  DT := FormatDateTime('dd"/"mm"/"yy', ADate);
  Result := System.SysUtils.Format(API_AGENT_STATION_PASSWORD, [AConfiguration, APlatform, AName, AUser, DT]);
end;

function TSabUri.URI_GetConfigurations: string;
begin
  Result := PRIVATE_API_AGENT_CONFIGS_PATH;
end;

function TSabUri.URI_GetDelUpdateRecord(const ARecId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_RECORD_PATH, [ARecId]);
end;

function TSabUri.URI_GetErrorMessages(ALanguage: TSabLanguage): string;
begin
  case ALanguage of
    sablngEnglish:  Result := API_MESSAGES_EN;
    sablngCzech:    Result := API_MESSAGES_CS;
    sablngGermany:  Result := API_MESSAGES_DE;
    sablngFrance:   Result := API_MESSAGES_FR;
  end;
end;

function TSabUri.URI_GetExternalFileParams(const ABoxId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_SAFEBOX_GEN_EXT_PARAMS, [ABoxId]);
end;

function TSabUri.URI_GetFileSizeLimit: string;
begin
  Result := API_FILE_SIZE_LIMIT;
end;

function TSabUri.URI_GetPassword(const AName, AConfiguration, APlatform, AUser: string; ADate: TDateTime): string;
var
  DT: string;
begin
  DT := FormatDateTime('dd"/"mm"/"yy', ADate);
  Result := System.SysUtils.Format(API_STATION_PASSWORD, [AConfiguration, APlatform, AName, AUser, DT]);
end;

function TSabUri.URI_GetSecondFactorQrImage(const APath: string): string;
begin
  Result := System.SysUtils.Format(API_GET_2ND_FACTOR_QR_IMAGE, [APath]);
end;

function TSabUri.URI_GetStations(const AConfiguration: string): string;
begin
  Result := System.SysUtils.Format(API_CONFIG_STATIONS, [AConfiguration]);
end;

function TSabUri.URI_GetStatus: string;
begin
  Result := GET_STATUS;
end;

function TSabUri.URI_GetTicketPassword(const ATicketId, AStation, APlatform, AUser: string): string;
begin
  Result := System.SysUtils.Format(API_GET_TICKET_PASSWORD, [ATicketId, AStation, APlatform, AUser]);
end;

function TSabUri.URI_GetUpdateRecordPasswords(const ARecId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_RECORD_PASSWORDS_PATH, [ARecId]);
end;

function TSabUri.URI_GetUserConfiguration(const ACode: string): string;
begin
  if ACode <> '' then
    Result := System.SysUtils.Format(API_GET_CONFIGURATION_CODE, [ACode])
  else
    Result := API_GET_CONFIGURATION;
end;

function TSabUri.URI_Login: string;
begin
  Result := API_LOGIN_PATH;
end;

function TSabUri.URI_LoginStatus: string;
begin
  Result := API_LOGIN_STATUS_PATH;
end;

function TSabUri.URI_ReadRemoveRestoreDeletedRecords: string;
begin
  Result := API_GET_DELETED_RECORDS;
end;

function TSabUri.URI_ReadDeleteSafeBox(const ABoxId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_SAFEBOX_PATH, [ABoxId]);
end;

function TSabUri.URI_ReadSafeBoxWithExtPath(const ABoxId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_SAFEBOX_GET_EXT_PATH, [ABoxId]);
end;

function TSabUri.URI_ReadSecureAttrs(ARecId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_RECORD_FORM_PATH, [ARecId]);
end;

function TSabUri.URI_RegisterSecondFactor: string;
begin
  Result := API_USER_REGISTER_2ND_FACTOR_PATH;
end;

function TSabUri.URI_ReadRecord(const ARecId: TSabId): string;
begin
  Result := URI_GetDelUpdateRecord(ARecId);
end;

function TSabUri.URI_ReadRecordForm(const ARecId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_RECORD_FORM_PATH, [ARecId])
end;

function TSabUri.URI_ReadRecordPasswords(const ARecId: TSabId): string;
begin
  Result := URI_GetUpdateRecordPasswords(ARecId);
end;

function TSabUri.URI_ReadRecords(const ABoxId: TSabId; const GetFileSizes: Boolean): string;
begin
  if GetFileSizes then
    Result := System.SysUtils.Format(API_SAFEBOX_RECORDS_PATH_FILE_SIZES, [ABoxId])
  else
    Result := System.SysUtils.Format(API_SAFEBOX_RECORDS_PATH, [ABoxId]);
end;

function TSabUri.URI_ReadSafeBoxes(const AParentBoxId: TSabId): string;
begin
  if AParentBoxId > 0 then
    Result := System.SysUtils.Format(API_SAFEBOX_BOXES_PATH, [AParentBoxId])
  else
    Result := API_SAFE_BOXES_PATH;
end;

function TSabUri.URI_Search(const APattern: string): string;
begin
  Result := System.SysUtils.Format(API_SAFE_SEARCH_PATH, [APattern]);
end;

function TSabUri.URI_SearchByNote(const APattern: string): string;
begin
  Result := System.SysUtils.Format(API_SAFE_SEARCH_BY_NOTE_PATH, [APattern]);
end;

function TSabUri.URI_SecondFactor: string;
begin
  Result := API_USER_2ND_FACTOR_PATH;
end;

function TSabUri.URI_SetPasswordToInvitedUser(const ACode: string): string;
begin
  Result := System.SysUtils.Format(API_SET_PASSWORD_TO_INVITED_USER, [ACode]);
end;

function TSabUri.URI_SetUserNewAccessCode: string;
begin
  Result := API_USER_NEW_ACCESSCODE;
end;

function TSabUri.URI_SetWatched(const ABoxId: TSabId; AWatchType: TWatchedType; All, Enable: Boolean): string;
var
  Frmt: string;
begin
  if AWatchType = wtChanges then
    if Enable then
      if All then
        Frmt := SAFE_WATCH_ENABLE_CHANGES_ALL
      else
        Frmt := SAFE_WATCH_ENABLE_CHANGES
    else
      if All then
        Frmt := SAFE_WATCH_DISABLE_CHANGES_ALL
       else
        Frmt := SAFE_WATCH_DISABLE_CHANGES
  else
    if Enable then
      if All then
        Frmt := SAFE_WATCH_ENABLE_ACCESSES_ALL
      else
        Frmt := SAFE_WATCH_ENABLE_ACCESSES
    else
      if All then
        Frmt := SAFE_WATCH_DISBLE_ACCESSES_ALL
      else
        Frmt := SAFE_WATCH_DISBLE_ACCESSES;

  Result := System.SysUtils.Format(Frmt, [ABoxId]);
end;

function TSabUri.URI_UpdateRecord(const ARecId: TSabId): string;
begin
  Result := URI_GetDelUpdateRecord(ARecId);
end;

function TSabUri.URI_UpdateSafeBox(const ABoxId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_SAFEBOX_PATH, [ABoxId]);
end;

function TSabUri.URI_UploadFileToRecord(ARecId: TSabId): string;
begin
  Result := System.SysUtils.Format(API_FILEREC_UPLOAD, [ARecId]);
end;

end.
