
{*******************************************************}
{                                                       }
{       SAB Object Framework                            }
{       SAB Types                                       }
{                                                       }
{       Copyright (c) 2017-22 TDP                       }
{                                                       }
{****************************************************tk*}

unit SAB.Types;

{ DEFINE LONG_ID}

interface

uses
  TDP.IntList;

type
{$IFDEF LONG_ID}
  TSabId = Int64;
  TSabIdList = TInt64List;
{$ELSE}
  TSabId = Integer;
  TSabIdList = TIntegerList;
{$ENDIF}

  TSabObjectType = (sabSafeBox, sabSafeBoxGroup, sabAccount, sabAccountSec, sabFile, sabCertificate, sabCreditCard,
    sabStation, sabConfiguration, sabExternalFile);
  TSabSafeBoxType = sabSafeBox..sabSafeBoxGroup;

  TSabObjectSet = set of TSabObjectType;

  TSabPlatform = (apWindows, spLinux, spMac, spLDAP);
  TSabPlatforms = set of TSabPlatform;

  TSabLanguage = (sablngEnglish, sablngCzech, sablngGermany, sablngFrance);

  TSabLauncher = (sabLauncherNone, sabRemoteDesktop, sabSSH, sabSSHFile, sabSecureProt, sabTelnet, sabFTP,
    sabRasdial, sabMikrotikWinbox);

  TCryptoStatusEvent = function(const APerc: Integer): Boolean of object;

const
  ANSWER_OK = 'OK';

  EXTFILE_PREPOSTFIX = SizeOf(UInt32) + SizeOf(UInt32) + 16;  // sign + rec-id + CRC tab

  SabLanguages: array[TSabLanguage] of string = ('en', 'cs', 'de', 'fr');

  SabObjectName: array[TSabObjectType] of string = (
    'Safe Box',       // sabSafeBox,
    'Safe Box Group', // sabSafeBoxGroup,
    'Account',        // sabAccount,
    'Secret Account', // sabAccountSec,
    'File',           // sabFile,
    'Certificate',    // sabCertificate,
    'Credit Card',    // sabCreditCard,
    'Station',        // sabStation,
    'Configuration',  // sabConfiguration
    'External File'   // sabExternalFile
    );


  SabLauncherSign: array[TSabLauncher] of string = (
    'NONE',
    'REMOTE_DESKTOP',
    'SSH',
    'SFTP',
    'SCP',
    'TELNET',
    'FTP',
    'RASDIAL',
    'WINBOX'
    );

  SabLauncherDesc: array[TSabLauncher] of string = (
    'n/a',
    'Remote Desktop',
    'SSH',
    'SSH file transfer protocol',
    'Secure Copy Protocol',
    'Telnet',
    'FTP',
    'Rasdial (VPN)',
    'MikroTik Winbox'
    );

  PasswordQualityByEntropy: array[0..5] of Double = (
     0.00, // prilis kratke/too short
     3.00, // velmi slabe/very week
    20.00, // slabe/weak
    35.00, // dobre/good
    40.00, // silne/strong
    50.00  // velmi silne/very strong
    );

  SabBoxes = [sabSafeBox];

  AllSabObjects = [sabSafeBox, sabSafeBoxGroup, sabAccount, sabAccountSec, sabFile, sabCertificate, sabCreditCard, sabStation,
    sabConfiguration, sabExternalFile];
  FileSabObjectsSet = [sabFile, sabCertificate];

  AllSabObjectsOrExternalFile = [sabSafeBox, sabSafeBoxGroup, sabAccount, sabAccountSec, sabFile, sabCertificate, sabCreditCard, sabStation,
    sabConfiguration, sabExternalFile];
  FileSabObjectsOrExternalFileSet = [sabFile, sabCertificate, sabExternalFile];

  AllSabBoxes = [sabSafeBox, sabSafeBoxGroup];
  AllSabRecords = [sabAccount, sabAccountSec, sabFile, sabCertificate, sabCreditCard, sabStation, sabConfiguration, sabExternalFile];

function StrToLauncher(const Sign: string): TSabLauncher;

implementation

uses
  System.SysUtils;

function StrToLauncher(const Sign: string): TSabLauncher;
begin
  for Result := Low(SabLauncherSign) to High(SabLauncherSign) do
    if SameText(Sign, SabLauncherSign[Result]) then
      Exit;

  Result := sabLauncherNone;
end;

end.
