
{*******************************************************}
{                                                       }
{       TDP-Ontrack Data Recovery Units                 }
{                                                       }
{       Copyright (c) 1994-2022 TDP                     }
{       Copyright (c) 2004-2018 TDP-Ontrack             }
{                                                       }
{****************************************************tk*}

unit TDP.CommonCode;

{$IFNDEF VER200}
  {$EXCESSPRECISION OFF}
{$ENDIF}

interface

uses
  WinApi.Windows, System.SysUtils, System.Classes,
  TDP.CommonTypes;

const
  MAX_INT64 = 9223372036854775807;

type
{ TStringTokenizer }

  TStringTokenizer = class(TObject)
  private
    FString: string;
    FSeparator: string;
    FOldPos: Integer;
    FPos: Integer;

    function GetCount: Integer;
    function GetItem(Index: Integer): string;
    procedure Init;
    procedure SetSeparator(const Value: string);
    procedure SetStr(const Value: string);
    function GetNumber(Index: Integer): Integer;
    function GetReal(Index: Integer): Extended;
    function GetLong(Index: Integer): Int64;
  public
    constructor Create(const ASeparator: string = ''); overload;
    constructor Create(const AString, ASeparator: string); overload;

    function HasMore: Boolean;
    function GetNext: string;

    property Count: Integer read GetCount;
    property Items[Index: Integer]: string read GetItem; default; // indexes 0..n
    property Numbers[Index: Integer]: Integer read GetNumber;
    property Reals[Index: Integer]: Extended read GetReal;
    property Longs[Index: Integer]: Int64 read GetLong;
    property Str: string read FString write SetStr;
    property Separator: string read FSeparator write SetSeparator;
  end;

{ TDestFileStream class }

  TDestFileStream = class(THandleStream)
  strict private
    FFileName: string;
  public
    constructor Create(const AFileName: string; AMode: Word; const CreateMode: Word = 0); overload;
    destructor Destroy; override;

    procedure Close;

    property FileName: string read FFileName;
  end;

{ TDestTextWriter class }

  TDstTextWriter = class(TStreamWriter)
  private
    FStream: TDestFileStream;
    FFileName: string;

    function GetStrWriter: Boolean;
  public
    constructor Create(const AFilename: string; AEncoding: TEncoding = nil; Append: Boolean =  False; BufferSize: Integer = 1024); overload;
    constructor Create(const AStream: TStringStream); overload;
    destructor Destroy; override;

    property StrWriter: Boolean read GetStrWriter;
    property Stream: TDestFileStream read FStream;
    property Filename: string read FFilename;
  end;

{ TAsciiTextReader class }

  TAsciiTextReader = class(TStreamReader)
  private
    FStream: TFileStream;
    FFileName: string;
  public
    constructor Create(const AFilename: string; BufferSize: Integer = 4096);
    destructor Destroy; override;

    class function Reset(Reader: TAsciiTextReader): TAsciiTextReader;

    property Stream: TFileStream read FStream;
    property Filename: string read FFilename;
  end;

  TDestTextWriter = class(TDstTextWriter)
  private
    FStream: TDestFileStream;

    FBuffer: TBytes;
    FBufferSize: Integer;
    FBufferIndex: Integer;

    procedure CheckBuffer;
    procedure MyWriteBytes(Bytes: TBytes);
  public
    constructor Create(const AFilename: string; AEncoding: TEncoding = nil; Append: Boolean =  False; BufferSize: Integer = 1024);
    destructor Destroy; override;

    procedure Close; override;
    procedure Flush; override;

    // Has own buffer. Flush method must be performed before using other methods
    procedure Write(const Value: String8; const PreFlush: Boolean = False); overload;
    procedure WriteLine(const Value: String8; const PreFlush: Boolean = False); overload;
    procedure WriteFmt(const Format: string; Args: array of const; const PreFlush: Boolean = False);
    procedure WriteFmtLine(const Format: string; Args: array of const; const PreFlush: Boolean = False);

    property Stream: TDestFileStream read FStream;
  end;

{$IFDEF CONSOLE}
{ TStopWatch class }

  TDRStopWatchState = (swIdle, swRunning, swStopped);

  TDurationData = record
    Hour: Integer;
    Minute: Integer;
    Second: Integer;
  end;

  TStopWatch = class(TObject)
  private
    FState: TDRStopWatchState;
    FStartTime: TDateTime;
    FStopTime: TDateTime;

    FElapsed: TDateTime;

    FCurrent: Int64;
    FTotal: Int64;

    procedure SetCurrent(const Value: Int64);
    procedure SetTotal(const Value: Int64);
    function GetElapsed: TDurationData;
    function GetPercent: Double;
    function GetRemains: TDurationData;

  public
    procedure Start;
    procedure Stop;

    // vypocte hodnoty Elapsed, Remains a Percent
    procedure Update; overload;
    procedure Update(const ACurrent, ATotal: Int64); overload;

    property Current: Int64 read FCurrent write SetCurrent;
    property Total: Int64 read FTotal write SetTotal;

    property Elapsed: TDurationData read GetElapsed;
    property Percent: Double read GetPercent;
    property Remains: TDurationData read GetRemains;
  end;
{$ENDIF}

{$IF CompilerVersion < 20}
function CharInSet(C: AnsiChar; const CharSet: TSysCharSet): Boolean; overload; inline;
function CharInSet(C: WideChar; const CharSet: TSysCharSet): Boolean; overload; inline;
{$IFEND}

function AnsiStrToOem(const AnsiStr: AnsiString): AnsiString;
function OemToAnsiStr(const OemStr: AnsiString): AnsiString;
{$IF CompilerVersion >= 20}
function StrToOem(const Str: string): string;
function OemToStr(const OemStr: string): string;
{$IFEND}
function StrToPrint(const S: string): string; overload;
function StrToPrint(const S: RawByteString): string; overload;
function EncodePrintStr(const S: string): RawByteString;

{$IFDEF CONSOLE}
procedure ResetPropeller;
function Propeller: Char;
{$ENDIF}

function Div2(A, B: Int64): Int64; overload;
function Div2(A, B: Integer): Integer; overload;

{ Swaping}
function BEToLE(const I: Byte): Byte; overload;
function BEToLE(const I: Shortint): Shortint; overload;
function BEToLE(const I: Smallint): Smallint; overload;
function BEToLE(const I: Word): Word; overload;
function BEToLE(const I: Integer): Integer; overload;
function BEToLE(const I: Cardinal): Cardinal; overload;
function BEToLE(const I: Int64): Int64; overload;
function BEToLE(const Ch: Char): Char; overload;
procedure BEToLE(const Str: PChar; ALen: Integer); overload;

// extract filename without extension
function ExtractPureFileName(const FileName: string): string;

// Variants
function UltimateNull(const AVar: OleVariant): Boolean;

// RemoveExtraSpaces
function RemoveExtraSpaces(const S: string): string;

{ Find pattern in byte buffer }

function BytePos(const Pattern: TBytes; const Buffer; const BufLen: Integer;
  AForward: Boolean = True; AStart: Integer = -1): Integer;

{ Safe clear of string }

procedure WipeString(var S: string);
procedure WipeAnsiChar(const P: PAnsiChar; const Len: Integer);

implementation

uses
  System.Math, System.DateUtils, System.StrUtils, System.Variants;

{$IF CompilerVersion < 20}
function CharInSet(C: AnsiChar; const CharSet: TSysCharSet): Boolean;
begin
  Result := C in CharSet;
end;

function CharInSet(C: WideChar; const CharSet: TSysCharSet): Boolean;
begin
  Result := (C < #$0100) and (AnsiChar(C) in CharSet);
end;
{$IFEND}

function AnsiStrToOem(const AnsiStr: AnsiString): AnsiString;
begin
  Result := AnsiStr;
  if Length(Result) > 0 then
    CharToOemA(PAnsiChar(Result), PAnsiChar(Result));
end;

function OemToAnsiStr(const OemStr: AnsiString): AnsiString;
begin
  Result := OemStr;
  if Length(Result) > 0 then
    OemToCharA(PAnsiChar(Result), PAnsiChar(Result));
end;

{$IF CompilerVersion >= 20}
function StrToOem(const Str: string): string;
var
  S: RawByteString;
begin
  Result := Str;
  if Length(Result) > 0 then
  begin
    CharToOemW(PChar(Result), PAnsiChar(S));
    Result := string(S);
  end;
end;

function OemToStr(const OemStr: string): string;
var
  S: AnsiString;
begin
  Result := OemStr;
  if Length(Result) > 0 then
  begin
    S := AnsiString(Result);
    OemToCharW(PAnsiChar(S), PChar(Result));
  end;
end;
{$IFEND}

function StrToPrint(const S: string): string;
begin
  Result := StrToPrint(UTF8Encode(S));
end;

function StrToPrint(const S: RawByteString): string;
var
  I: Integer;
  C: AnsiChar;
begin
  Result := '';
  for I := 1 to Length(S) do
  begin
    C := S[I];
    case C of
      '#':
        Result := Result + '##';
      #0..#31,
      #128..#255:
        Result := Result + '#' + IntToHex(Ord(C), 2);
    else
      Result := Result + Char(C);
    end;
  end;
end;

function EncodePrintStr(const S: string): RawByteString;
var
  N, Org: Integer;
  No: string;
  B: Integer;
begin
  Result := '';
  Org := 1;
  N := PosEx('#', S);
  while N > 0 do
  begin
    Result := Result + RawByteString(Copy(S, Org, N - Org));
    if N + 1 < Length(S) then
    begin
      if S[N + 1] = '#' then
      begin
        Result := Result + '#';
        Inc(N);
      end else
      if N + 2 < Length(S) then
      begin
        No := Copy(S, N + 1, 2);
        B := StrToIntDef('$' + No, -1);
        if B >= 0 then
        begin
          Result := Result + AnsiChar(B);
          Inc(N, 2);
        end else
          Result := Result + '#';
      end else
        Result := Result + '#';
    end else
      Result := Result + '#';

    Inc(N);
    Org := N;
    N := PosEx('#', S, Org);
  end;
  if Org < Length(S) then
    Result := Result + RawByteString(Copy(S, Org, Length(S)));
end;

function UltimateNull(const AVar: OleVariant): Boolean;
begin
  Result := VarIsNull(AVar) or VarIsEmpty(AVar) or VarIsClear(AVar);
end;

{ ExtractPureFileName }

function ExtractPureFileName(const FileName: string): string;
var
  Ext: string;
begin
  Result := ExtractFileName(FileName);
  Ext := ExtractFileExt(FileName);
  Result := LeftStr(Result, Length(Result) - Length(Ext));
end;

{$IFDEF CONSOLE}
const
  SITES = 4;

var
  PropellerIdx: Integer = 0;

procedure ResetPropeller;
begin
  PropellerIdx := 0;
end;

function Propeller: Char;
const
  PropellerLeaves: array [0..SITES - 1] of Char = ('|', '/', '-', '\');
begin
  Result := PropellerLeaves[PropellerIdx];
  PropellerIdx := (PropellerIdx + 1) mod SITES;
end;
{$ENDIF}

{ TStringTokenizer }

constructor TStringTokenizer.Create(const AString, ASeparator: string);
begin
  inherited Create;
  FString := AString;
  FSeparator := ASeparator;
  Init;
end;

constructor TStringTokenizer.Create(const ASeparator: string);
begin
  inherited Create;
  FSeparator := ASeparator;
  Init;
end;

function TStringTokenizer.GetCount: Integer;
begin
  Result := 0;
  Init;
  while HasMore do
  begin
    Inc(Result);
    GetNext;
  end;
end;

function TStringTokenizer.GetItem(Index: Integer): string;
var
  S: string;
begin
  Result := '';
  Init;
  while HasMore do
  begin
    S := GetNext;
    Dec(Index);
    if Index < 0 then
      Exit(S);
  end;
end;

function TStringTokenizer.GetLong(Index: Integer): Int64;
var
  S: string;
begin
  Result := -1;
  Init;
  while HasMore do
  begin
    S := GetNext;
    Dec(Index);
    if Index < 0 then
      Exit(StrToInt64Def(Trim(AnsiDequotedStr(S, '"')), -MaxInt));
  end;
end;

function TStringTokenizer.GetNext: string;
begin
  FOldPos := FPos + 1;
  FPos := PosEx(FSeparator, FString, FOldPos);
  if FPos = 0 then
  begin
    Result := Trim(Copy(FString, FOldPos, Length(FString)));
    FOldPos := Length(FString) + 1;
  end else
  begin
    Result := Trim(Copy(FString, FOldPos, FPos - FOldPos));
    FOldPos := FPos + 1;
  end;
end;

function TStringTokenizer.GetNumber(Index: Integer): Integer;
var
  S: string;
begin
  Result := -1;
  Init;
  while HasMore do
  begin
    S := GetNext;
    Dec(Index);
    if Index < 0 then
      Exit(StrToIntDef(Trim(AnsiDequotedStr(S, '"')), -MaxInt));
  end;
end;

function TStringTokenizer.GetReal(Index: Integer): Extended;
var
  S: string;
begin
  Result := -1;
  Init;
  while HasMore do
  begin
    S := GetNext;
    Dec(Index);
    if Index < 0 then
      Exit(StrToFloatDef(Trim(AnsiDequotedStr(S, '"')), MinDouble));
  end;
end;

function TStringTokenizer.HasMore: Boolean;
begin
  Result := (FOldPos <= Length(FString)) and (FString <> '');
end;

procedure TStringTokenizer.Init;
begin
  FOldPos := 0;
  FPos := 0;
end;

procedure TStringTokenizer.SetSeparator(const Value: string);
begin
  FSeparator := Value;
  if (FSeparator = ' ') and (FString <> '') then
    FString := RemoveExtraSpaces(FString);
  Init;
end;

procedure TStringTokenizer.SetStr(const Value: string);
begin
  if Separator = ' ' then
    FString := RemoveExtraSpaces(Value)
  else
    FString := Value;
  Init;
end;

{ TDestFileStream }

const
  AccessMode: array[0..2] of LongWord = (
    GENERIC_READ,
    GENERIC_WRITE,
    GENERIC_READ or GENERIC_WRITE);

  ShareMode: array[0..4] of LongWord = (
    0,
    0,
    FILE_SHARE_READ,
    FILE_SHARE_WRITE,
    FILE_SHARE_READ or FILE_SHARE_WRITE);

constructor TDestFileStream.Create(const AFileName: string; AMode: Word; const CreateMode: Word);
var
  Mode: Word;
begin
  if AMode = fmCreate then
    Mode := CreateMode and $F0 or fmOpenWrite
  else
    Mode := AMode;

  Mode := Mode or fmShareDenyWrite;

  inherited Create(CreateFile(PChar(AFileName), AccessMode[Mode and 3],
    ShareMode[(Mode and $F0) shr 4], nil,
    IfThen(AMode = fmCreate, CREATE_ALWAYS, OPEN_EXISTING), FILE_ATTRIBUTE_NORMAL, 0));

  if FHandle = INVALID_HANDLE_VALUE then
    raise EFCreateError.CreateFmt('Nemohu otevrit/vytvorit soubor %s (%s)', [ExpandFileName(AFileName), SysErrorMessage(GetLastError)]);

  FFileName := AFileName;
end;

destructor TDestFileStream.Destroy;
begin
  if Longint(FHandle) >= 0 then FileClose(FHandle);
  inherited;
end;

procedure TDestFileStream.Close;
begin
  if FHandle <> INVALID_HANDLE_VALUE then
  begin
    FileClose(Handle);
    FHandle := INVALID_HANDLE_VALUE;
  end;
end;

{ TDstTextWriter }

constructor TDstTextWriter.Create(const AFilename: string; AEncoding: TEncoding; Append: Boolean; BufferSize: Integer);
begin
  if AFileName <> '' then
  begin
    if (not FileExists(AFilename)) or (not Append) then
      FStream := TDestFileStream.Create(AFilename, fmCreate)
    else
    begin
      FStream := TDestFileStream.Create(AFilename, fmOpenWrite);
      FStream.Seek(0, soEnd);
    end;
  end else
    FStream := TDestFileStream.Create(THandle(STD_OUTPUT_HANDLE));
  if not Assigned(AEncoding) then
    AEncoding := TEncoding.UTF8;

  FFileName := AFileName;
  inherited Create(FStream, AEncoding, BufferSize);
end;

constructor TDstTextWriter.Create(const AStream: TStringStream);
begin
  inherited Create(AStream);
end;

destructor TDstTextWriter.Destroy;
begin
  inherited;
  FStream.Free;
end;

function TDstTextWriter.GetStrWriter: Boolean;
begin
  Result := BaseStream is TStringStream;
end;

{ TAsciiTextReader }

constructor TAsciiTextReader.Create(const AFilename: string; BufferSize: Integer);
begin
  FStream := TFileStream.Create(AFilename, fmOpenRead + fmShareDenyWrite);
  FFileName := AFileName;
  inherited Create(FStream, TEncoding.ASCII, False, BufferSize);
end;

destructor TAsciiTextReader.Destroy;
begin
  inherited;
  FStream.Free;
end;

class function TAsciiTextReader.Reset(Reader: TAsciiTextReader): TAsciiTextReader;
var
  FName: string;
begin
  FName := Reader.Filename;
  Reader.Free;

  Result := TAsciiTextReader.Create(FName);
end;

{ TDestTextWriter }

constructor TDestTextWriter.Create(const AFilename: string; AEncoding: TEncoding; Append: Boolean; BufferSize: Integer);
begin
  inherited Create(AFilename, AEncoding, Append, BufferSize);
  FBufferSize := BufferSize;
end;

destructor TDestTextWriter.Destroy;
begin
  SetLength(FBuffer, 0);
  inherited;
end;

procedure TDestTextWriter.CheckBuffer;
begin
  if FBuffer = nil then
  begin
    if FBufferSize <= 0 then
      FBufferSize := 1024;

//    FBuffer := System.AllocMem(FBufferSize);
    SetLength(FBuffer, FBufferSize);
    FBufferIndex := 0;
  end;
end;

procedure TDestTextWriter.Close;
begin
  if Assigned(FStream) then
    FileClose(FStream.Handle);
  inherited;
end;

procedure TDestTextWriter.Flush;
begin
  if (FBuffer <> nil) and (FBufferIndex  <> 0) and (BaseStream <> nil) then
  begin
    BaseStream.Write(FBuffer[0], FBufferIndex);
    FBufferIndex := 0;
  end;
  inherited;
end;

procedure TDestTextWriter.MyWriteBytes(Bytes: TBytes);
var
  ByteIndex: Integer;
  WriteLen: Integer;
begin
  CheckBuffer;
  ByteIndex := 0;

  while ByteIndex < Length(Bytes) do
  begin
    WriteLen := Length(Bytes) - ByteIndex;
    if WriteLen > FBufferSize - FBufferIndex then
      WriteLen := FBufferSize - FBufferIndex;

    Move(Bytes[ByteIndex], FBuffer[FBufferIndex], WriteLen);

    Inc(FBufferIndex, WriteLen);
    Inc(ByteIndex, WriteLen);

    if FBufferIndex >= FBufferSize then
      Flush;
  end;

  if AutoFlush then
    Flush;
end;

procedure TDestTextWriter.Write(const Value: String8; const PreFlush: Boolean);
begin
  if PreFlush then
    Flush;

  MyWriteBytes(TBytes(Value));
end;

procedure TDestTextWriter.WriteFmt(const Format: string; Args: array of const;
  const PreFlush: Boolean);
var
  S: string8;
begin
  if PreFlush then
    Flush;

{$IFDEF VER200}
  S := String8(SysUtils.Format(Format, Args));
{$ELSE}
  S := String8(System.SysUtils.Format(Format, Args));
{$ENDIF}
  Write(S, PreFlush);
end;

procedure TDestTextWriter.WriteFmtLine(const Format: string; Args: array of const; const PreFlush: Boolean);
begin
  WriteFmt(Format + NewLine, Args, PreFlush);
end;

procedure TDestTextWriter.WriteLine(const Value: String8; const PreFlush: Boolean);
var
  S: string8;
begin
  if PreFlush then
    Flush;

  S := String8(string(Value) + NewLine);
  MyWriteBytes(TBytes(S));
end;

{$IFDEF CONSOLE}

{ TStopWatch }

function TStopWatch.GetElapsed: TDurationData;
var
  Y, L, D, H, M, S, W: Word;
begin
  if FState = swRunning then
  begin
    DecodeDateTime(FElapsed, Y, L, D, H, M, S, W);
    Result.Hour := H + (D - 30) * 24;
    Result.Minute := M;
    Result.Second := S;
  end else
    FillChar(Result, Sizeof(Result), 0);
end;

function TStopWatch.GetPercent: Double;
begin
  if Total <= 0 then
    Result := 100
  else
    Result := RoundTo(Current / Total * 100, -2);
end;

function TStopWatch.GetRemains: TDurationData;
var
  Remained, MinuteSecond: Int64;
  Y, L, D, H, M, S, W: Word;
  ElapsedSec: Int64;
begin
  if (FState = swRunning) and (Total >= 0) then
  begin
    DecodeDateTime(FElapsed, Y, L, D, H, M, S, W);
    ElapsedSec := Int64(H) * 3600 + Int64(M) * 60 + Int64(S);

    Remained := Round(ElapsedSec/(Current/Total) - ElapsedSec);
    MinuteSecond := Remained mod 3600;

    Result.Hour := Remained div 3600;
    Result.Minute := MinuteSecond div 60;
    Result.Second := MinuteSecond mod 60
  end else
    FillChar(Result, Sizeof(Result), 0);
end;

procedure TStopWatch.SetCurrent(const Value: Int64);
begin
  if Value > Total then
    FCurrent := Total
  else
    FCurrent := Value;
end;

procedure TStopWatch.SetTotal(const Value: Int64);
begin
  if Value < Current then
    FTotal := Current
  else
    FTotal := Value;
end;

procedure TStopWatch.Start;
begin
  FState := swRunning;
  FElapsed := 0;
  FStopTime := 0;
  FStartTime := Now();
end;

procedure TStopWatch.Stop;
begin
  FStopTime := Now();
  FState := swStopped;
end;

procedure TStopWatch.Update;
begin
  if FState <> swRunning then
    Start;

  FElapsed := Now() - FStartTime;
end;

procedure TStopWatch.Update(const ACurrent, ATotal: Int64);
begin
  Current := ACurrent;
  Total := ATotal;
  Update;
end;

{$ENDIF}

{ Swaping }

function BEToLE(const I: Byte): Byte;
begin
  Result := I;
end;

function BEToLE(const I: Shortint): Shortint;
begin
  Result := I;
end;

function BEToLE(const I: Smallint): Smallint;
begin
  Result := Swap(I);
end;

function BEToLE(const I: Word): Word;
begin
  Result := Swap(I);
end;

function BEToLE(const I: Integer): Integer;
begin
  LongRec(Result).Lo := Swap(LongRec(I).Hi);
  LongRec(Result).Hi := Swap(LongRec(I).Lo);
end;

function BEToLE(const I: Cardinal): Cardinal;
begin
  LongRec(Result).Lo := Swap(LongRec(I).Hi);
  LongRec(Result).Hi := Swap(LongRec(I).Lo);
end;

function BEToLE(const I: Int64): Int64;
begin
  Int64Rec(Result).Lo := BEToLE(Int64Rec(I).Hi);
  Int64Rec(Result).Hi := BEToLE(Int64Rec(I).Lo);
end;

function BEToLE(const Ch: Char): Char;
begin
  Result := Char(Swap(Word(Ch)));
end;

procedure BEToLE(const Str: PChar; ALen: Integer);
var
  I: Integer;
begin
  for I := 0 to ALen - 1 do
    Str[I] := BEToLE(Str[I]);
end;

{ Remove Extra Spaces }

function RemoveExtraSpaces(const S: string): string;
var
  I, Len: Integer;
  Spc, First: Boolean;
begin
  SetLength(Result, Length(S));
  Spc := False;
  First := True;
  Len := 0;
  for I := 1 to Length(S) do
  begin
    if S[I] <> ' ' then
    begin
      if Spc and not First then
      begin
        Inc(Len);
        Result[Len] := ' ';
      end;
      Inc(Len);
      Result[Len] := S[I];
      Spc := False;
      First := False;
    end else
      Spc := True;
  end;
  SetLength(Result, Len);
end;

function Div2(A, B: Int64): Int64;
begin
  Result := A div B;
  if A mod B > 0 then
    Inc(Result);
end;

function Div2(A, B: Integer): Integer;
begin
  Result := A div B;
  if A mod B > 0 then
    Inc(Result);
end;

{ Find pattern in byte buffer }

function BytePos(const Pattern: TBytes; const Buffer; const BufLen: Integer; AForward: Boolean; AStart: Integer): Integer;
var
  Buf: PByte;
  FirstByte: Byte;
  PatLen: Integer;
  I, J: Integer;
  OK: boolean;
begin
  Result := -1;
  Buf := @Buffer;

  PatLen := Length(Pattern);
  if PatLen > BufLen then Exit;
  if PatLen = 0 then Exit;

  FirstByte := Pattern[0];
  if AForward then
  begin
    if AStart < 0 then
      AStart := 0
    else
      if AStart >= BufLen - PatLen then Exit;

    for I := AStart to BufLen - PatLen do
    begin
      if Buf[I] = FirstByte then
      begin
        OK := True;
        for J := 1 to PatLen - 1 do
        begin
          if Buf[I + J] <> Pattern[J] then
          begin
            OK := False;
            Break
          end;
        end;
        if OK then
          Exit(I);
      end;
    end;
  end else
  begin
    if AStart < 0 then
      AStart := BufLen - PatLen
    else
      if AStart >= BufLen - PatLen then Exit;

    for I := AStart downto 0 do
    begin
      if Buf[I] = FirstByte then
      begin
        OK := True;
        for J := 1 to PatLen - 1 do
        begin
          if Buf[I + J] <> Pattern[J] then
          begin
            OK := False;
            Break;
          end;
        end;
        if OK then
          Exit(I);
      end;
    end;
  end;
end;

{ Safe clear of string }

procedure WipeString(var S: string);
begin
  UniqueString(S);
  ZeroMemory(Pointer(S), Length(S) * SizeOf(Char));
  S := '';
end;

procedure WipeAnsiChar(const P: PAnsiChar; const Len: Integer);
begin
  ZeroMemory(P, Len);
end;

end.
